@@ -1,6 +1,6 @@
 #  MIT License
 #
-#  Copyright (c) 2017-2021 TileDB Inc.
+#  Copyright (c) 2017-2022 TileDB Inc.
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -51,7 +51,7 @@ save_return_as_preference <- function(value = c("asis", "array", "matrix", "data
     cfgdir <- tools::R_user_dir(packageName())
     if (!dir.exists(cfgdir)) dir.create(cfgdir)
     fname <- file.path(cfgdir, "config.dcf")
-    con <- file(fname, "w+")
+    con <- file(fname, "a+")
     cat("return_as:", value, "\n", file=con)
     close(con)
     set_return_as_preference(value)
@@ -79,11 +79,86 @@ get_return_as_preference <- function() .pkgenv[["return_as"]]
 ##' @rdname save_return_as_preference
 ##' @export
 set_return_as_preference <- function(value = c("asis", "array", "matrix", "data.frame",
-                                                          "data.table", "tibble")) {
+                                               "data.table", "tibble")) {
     value <- match.arg(value)
     .pkgenv[["return_as"]] <- value
 }
 
+
+##' Save (or load) allocation size default preference in an optional
+##' config file
+##'
+##' When retrieving data from sparse arrays, allocation sizes cannot
+##' be determined \emph{ex ante} as the degree of sparsity is unknown.
+##' A configuration value can aide in providing an allocation size
+##' value. These functions let the user store such a value for
+##' retrieval by the package code.  The preference will be enconded in
+##' a configuration file as R (version 4.0.0 or later) allows a user-
+##' and package specific configuration files.  These helper functions
+##' sets and retrieve the value, respectively, or retrieve the cached
+##' value from the package environment where is it set at package
+##' load.
+##'
+##' The value will be stored as a character value and reparsed so
+##' \sQuote{1e6} and \sQuote{1000000} are equivalent, and the fixed
+##' (but adjustable) number of digits for numerical precision
+##' \emph{use for formatting} will impact the writing. This should
+##' have no effect on standard allocation sizes.
+##'
+##' Note that this memory budget (currently, at least) applies only to
+##' character columns.  A fallback value of 50mb is used if no user
+##' value is set, and is used for all columns (rather than per column).
+##'
+##' @note This function requires R version 4.0.0 or later to utilise the per-user
+##' config directory accessor function. For older R versions, a fallback from the
+##' TileDB configuration object is used.
+##' @title Store allocation size preference
+##' @param value A numeric value with the desired allocation size (in bytes).
+##' @return For the setter, \code{TRUE} is returned invisibly but the function is invoked for the
+##' side effect of storing the value. For the getters, the value as a numeric.
+##' @export
+save_allocation_size_preference <- function(value) {
+    stopifnot(`This function relies on R version 4.0.0 or later.` = R.version.string >= "4.0.0",
+              `The 'value' has to be numeric` = is.numeric(value))
+
+    cfgdir <- tools::R_user_dir(packageName())
+    if (!dir.exists(cfgdir)) dir.create(cfgdir)
+    fname <- file.path(cfgdir, "config.dcf")
+    con <- file(fname, "a+")
+    cat("allocation_size:", value, "\n", file=con)
+    close(con)
+    set_allocation_size_preference(value)
+    invisible(TRUE)
+}
+
+##' @rdname save_allocation_size_preference
+##' @export
+load_allocation_size_preference <- function() {
+    value <- 50 * 1024 * 1024           # fallback value is 50mb
+    cfgfile <- .defaultConfigFile()     # but check config file
+    if (cfgfile != "" && file.exists(cfgfile)) {
+        cfg <- read.dcf(cfgfile)
+        if ("allocation_size" %in% colnames(cfg))
+            value <- as.numeric(cfg[[1, "allocation_size"]])
+    }
+    set_allocation_size_preference(value)
+    value
+}
+
+##' @rdname save_allocation_size_preference
+##' @export
+get_allocation_size_preference <- function() .pkgenv[["allocation_size"]]
+
+##' @rdname save_allocation_size_preference
+##' @export
+set_allocation_size_preference <- function(value) {
+    stopifnot(`The 'value' has to be numeric` = is.numeric(value))
+    .pkgenv[["allocation_size"]] <- value
+}
+
+
+
+
 is.scalar <- function(x, typestr) {
     (typeof(x) == typestr) && is.atomic(x) && length(x) == 1L
 }
